@@ -24,24 +24,6 @@ stderr = None
 logger = logging.getLogger('system_libs')
 
 
-def call_process(cmd):
-  shared.run_process(cmd, stdout=stdout, stderr=stderr)
-
-
-def run_commands(commands):
-  cores = min(len(commands), shared.Building.get_num_cores())
-  if cores <= 1:
-    for command in commands:
-      call_process(command)
-  else:
-    pool = shared.Building.get_multiprocessing_pool()
-    # https://stackoverflow.com/questions/1408356/keyboard-interrupts-with-pythons-multiprocessing-pool
-    # https://bugs.python.org/issue8296
-    # 999999 seconds (about 11 days) is reasonably huge to not trigger actual timeout
-    # and is smaller than the maximum timeout value 4294967.0 for Python 3 on Windows (threading.TIMEOUT_MAX)
-    pool.map_async(call_process, commands, chunksize=1).get(999999)
-
-
 def files_in_path(path_components, filenames):
   srcdir = shared.path_from_root(*path_components)
   return [os.path.join(srcdir, f) for f in filenames]
@@ -58,6 +40,28 @@ def get_cflags(force_object_files=False):
   return flags
 
 
+def run_build_command(cmd):
+  # this must only be called on a standard build command
+  assert cmd[0] == shared.PYTHON and cmd[1] in (shared.EMCC, shared.EMXX)
+  # add standard cflags, but also allow the cmd to override them
+  cmd = cmd[:2] + get_cflags() + cmd[2:]
+  shared.run_process(cmd, stdout=stdout, stderr=stderr)
+
+
+def run_commands(commands):
+  cores = min(len(commands), shared.Building.get_num_cores())
+  if cores <= 1:
+    for command in commands:
+      run_build_command(command)
+  else:
+    pool = shared.Building.get_multiprocessing_pool()
+    # https://stackoverflow.com/questions/1408356/keyboard-interrupts-with-pythons-multiprocessing-pool
+    # https://bugs.python.org/issue8296
+    # 999999 seconds (about 11 days) is reasonably huge to not trigger actual timeout
+    # and is smaller than the maximum timeout value 4294967.0 for Python 3 on Windows (threading.TIMEOUT_MAX)
+    pool.map_async(run_build_command, commands, chunksize=1).get(999999)
+
+
 def create_lib(libname, inputs):
   """Create a library from a set of input objects."""
   if libname.endswith('.bc'):
@@ -160,7 +164,7 @@ def calculate(temp_files, in_temp, stdout_, stderr_, forced=[]):
               '-Wno-empty-body']
     for src in files:
       o = in_temp(os.path.basename(src) + '.o')
-      commands.append([shared.PYTHON, shared.EMCC, shared.path_from_root('system', 'lib', src), '-o', o] + musl_internal_includes() + default_opts + c_opts + lib_opts + get_cflags())
+      commands.append([shared.PYTHON, shared.EMCC, shared.path_from_root('system', 'lib', src), '-o', o] + musl_internal_includes() + default_opts + c_opts + lib_opts)
       o_s.append(o)
     run_commands(commands)
     create_lib(in_temp(lib_filename), o_s)
@@ -180,7 +184,7 @@ def calculate(temp_files, in_temp, stdout_, stderr_, forced=[]):
     for src in files:
       o = in_temp(src + '.o')
       srcfile = shared.path_from_root(src_dirname, src)
-      commands.append([shared.PYTHON, shared.EMXX, srcfile, '-o', o, '-std=c++11'] + opts + get_cflags())
+      commands.append([shared.PYTHON, shared.EMXX, srcfile, '-o', o, '-std=c++11'] + opts)
       o_s.append(o)
     run_commands(commands)
     create_lib(in_temp(lib_filename), o_s)
@@ -207,11 +211,19 @@ def calculate(temp_files, in_temp, stdout_, stderr_, forced=[]):
   def gl_version_flags(libname):
     if shared.Settings.USE_WEBGL2:
       assert '-webgl2' in libname
-      return ['-DUSE_WEBGL2=1']
+      return ['-DUSE_WEBGL2=1', '-s', 'USE_WEBGL2=1']
     else:
       assert '-webgl2' not in libname
       return []
 
+  def gl_offscreen_flags(libname):
+    if shared.Settings.OFFSCREEN_FRAMEBUFFER:
+      assert '-ofb' in libname
+      return ['-D__EMSCRIPTEN_OFFSCREEN_FRAMEBUFFER__']
+    else:
+      assert '-ofb' not in libname
+      return []
+
   # libc
   def create_libc(libname):
     logger.debug(' building libc for cache')
@@ -416,10 +428,11 @@ def calculate(temp_files, in_temp, stdout_, stderr_, forced=[]):
     for dirpath, dirnames, filenames in os.walk(src_dir):
       filenames = filter(lambda f: f.endswith('.c'), filenames)
       files += map(lambda f: os.path.join(src_dir, f), filenames)
-    flags = ['-Oz', '-s', 'USE_WEBGL2=1']
+    flags = ['-Oz']
     flags += threading_flags(libname)
     flags += legacy_gl_emulation_flags(libname)
     flags += gl_version_flags(libname)
+    flags += gl_offscreen_flags(libname)
     return build_libc(libname, files, flags)
 
   # al
@@ -444,7 +457,7 @@ def calculate(temp_files, in_temp, stdout_, stderr_, forced=[]):
     commands = []
     for src in files:
       o = in_temp(os.path.basename(src) + '.o')
-      commands.append([shared.PYTHON, shared.EMCC, shared.path_from_root('system', 'lib', src), '-O2', '-o', o] + get_cflags())
+      commands.append([shared.PYTHON, shared.EMCC, shared.path_from_root('system', 'lib', src), '-O2', '-o', o])
       o_s.append(o)
     run_commands(commands)
     shared.Building.emar('cr', in_temp(libname), o_s)
@@ -552,6 +565,12 @@ def calculate(temp_files, in_temp, stdout_, stderr_, forced=[]):
                            filenames=['extras.c'])
     return create_wasm_rt_lib(libname, files)
 
+  def create_wasm_ubsan_minimal_rt(libname):
+    files = files_in_path(
+      path_components=['system', 'lib', 'compiler-rt', 'lib', 'ubsan_minimal'],
+      filenames=['ubsan_minimal_handlers.cpp'])
+    return create_wasm_rt_lib(libname, files)
+
   def create_wasm_libc_rt(libname):
     return create_wasm_rt_lib(libname, get_wasm_libc_rt_files())
 
@@ -654,6 +673,8 @@ def calculate(temp_files, in_temp, stdout_, stderr_, forced=[]):
     gl_name += '-emu'
   if shared.Settings.USE_WEBGL2:
     gl_name += '-webgl2'
+  if shared.Settings.OFFSCREEN_FRAMEBUFFER:
+    gl_name += '-ofb'
   system_libs += [Library(gl_name,        ext, create_gl,          gl_symbols,          [libc_name],   False)] # noqa
 
   if shared.Settings.USE_PTHREADS:
@@ -760,6 +781,9 @@ def calculate(temp_files, in_temp, stdout_, stderr_, forced=[]):
     libs_to_link.append((shared.Cache.get('libcompiler_rt_wasm.a', lambda: create_wasm_compiler_rt('libcompiler_rt_wasm.a')), False))
     libs_to_link.append((shared.Cache.get('libc_rt_wasm.a', lambda: create_wasm_libc_rt('libc_rt_wasm.a')), False))
 
+  if shared.Settings.UBSAN_RUNTIME == 1:
+    libs_to_link.append((shared.Cache.get('libubsan_minimal_rt_wasm.a', lambda: create_wasm_ubsan_minimal_rt('libubsan_minimal_rt_wasm.a')), False))
+
   libs_to_link.sort(key=lambda x: x[0].endswith('.a')) # make sure to put .a files at the end.
 
   # libc++abi and libc++ *static* linking is tricky. e.g. cxa_demangle.cpp disables c++
@@ -813,7 +837,7 @@ class Ports(object):
     objects = []
     for src in srcs:
       obj = src + '.o'
-      commands.append([shared.PYTHON, shared.EMCC, '-c', src, '-O2', '-o', obj, '-w'] + include_commands + flags + get_cflags())
+      commands.append([shared.PYTHON, shared.EMCC, '-c', src, '-O2', '-o', obj, '-w'] + include_commands + flags)
       objects.append(obj)
 
     run_commands(commands)
